"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const path = require("path");
const bodyParser = require("body-parser");
const URL = require("url");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const typescript_rest_1 = require("typescript-rest");
const _ = require("lodash");
const config_1 = require("../config");
const ApiError_1 = require("./ApiError");
const MatrixSecurity_1 = require("./security/MatrixSecurity");
/**
 * Web server for Dimension. Handles the API routes for the admin, scalar, dimension, and matrix APIs.
 */
class Webserver {
    constructor() {
        this.app = express();
        this.configure();
        this.loadRoutes();
    }
    loadRoutes() {
        // TODO: Rename services to controllers, and controllers to services. They're backwards.
        const apis = ["scalar", "dimension", "admin", "matrix"].map(a => path.join(__dirname, a, "*.js"));
        const router = express.Router();
        typescript_rest_1.Server.useIoC();
        typescript_rest_1.Server.registerAuthenticator(new MatrixSecurity_1.default());
        apis.forEach(a => typescript_rest_1.Server.loadServices(router, [a]));
        const routes = _.uniq(router.stack.map(r => r.route.path));
        for (const route of routes) {
            this.app.options(route, (_req, res) => res.sendStatus(200));
            matrix_js_snippets_1.LogService.info("Webserver", "Registered route: " + route);
        }
        this.app.use(router);
        // We register the default route last to make sure we don't override anything by accident.
        // We'll pass off all other requests to the web app
        this.app.get(/(widgets\/|riot\/|element\/|\/)*/, (_req, res) => {
            res.sendFile(path.join(__dirname, "..", "..", "web", "index.html"));
        });
        // Set up the error handler
        this.app.use((err, _req, res, next) => {
            if (err instanceof ApiError_1.ApiError) {
                // Don't do anything for 'connection reset'
                if (res.headersSent)
                    return next(err);
                matrix_js_snippets_1.LogService.warn("Webserver", "Handling ApiError " + err.statusCode + " " + JSON.stringify(err.jsonResponse));
                res.setHeader("Content-Type", "application/json");
                res.status(err.statusCode);
                res.json(err.jsonResponse);
            }
            else
                next(err);
        });
    }
    configure() {
        this.app.use(express.static(path.join(__dirname, "..", "..", "web")));
        this.app.use(bodyParser.json({ limit: '512mb' }));
        this.app.use((req, _res, next) => {
            const parsedUrl = URL.parse(req.url, true);
            if (parsedUrl.query && parsedUrl.query["scalar_token"]) {
                parsedUrl.query["scalar_token"] = "redacted";
                parsedUrl.search = undefined; // to force URL.format to use `query`
            }
            if (parsedUrl.query && parsedUrl.query["access_token"]) {
                parsedUrl.query["access_token"] = "redacted";
                parsedUrl.search = undefined; // to force URL.format to use `query`
            }
            matrix_js_snippets_1.LogService.info("Webserver", "Incoming request: " + req.method + " " + URL.format(parsedUrl));
            next();
        });
        this.app.use((_req, res, next) => {
            if (res.headersSent) {
                next();
                return;
            }
            res.setHeader("Access-Control-Allow-Origin", "*");
            res.setHeader("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept, Authorization");
            next();
        });
    }
    /**
     * Starts the webserver, bootstrapping the various API handlers
     */
    start() {
        this.app.listen(config_1.default.web.port, config_1.default.web.address);
        matrix_js_snippets_1.LogService.info("Webserver", "API and UI listening on " + config_1.default.web.address + ":" + config_1.default.web.port);
    }
}
exports.default = Webserver;
//# sourceMappingURL=Webserver.js.map