"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const MatrixOpenIdClient_1 = require("../../matrix/MatrixOpenIdClient");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const ApiError_1 = require("../ApiError");
const User_1 = require("../../db/models/User");
const Upstream_1 = require("../../db/models/Upstream");
const ScalarStore_1 = require("../../db/ScalarStore");
const UserScalarToken_1 = require("../../db/models/UserScalarToken");
const ScalarClient_1 = require("../../scalar/ScalarClient");
const randomString = require("random-string");
const es6_1 = require("typescript-ioc/es6");
const MemoryCache_1 = require("../../MemoryCache");
/**
 * API controller for account management
 */
let AccountController = class AccountController {
    constructor() {
    }
    /**
     * Gets the owner of a given scalar token, throwing an ApiError if the token is invalid.
     * @param {string} scalarToken The scalar token to validate
     * @returns {Promise<string>} Resolves to the owner's user ID if the token is valid.
     * @throws {ApiError} Thrown with a status code of 401 if the token is invalid.
     */
    getTokenOwner(scalarToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const cachedUserId = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SCALAR_ACCOUNTS).get(scalarToken);
            if (cachedUserId)
                return cachedUserId;
            try {
                const user = yield ScalarStore_1.ScalarStore.getTokenOwner(scalarToken);
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SCALAR_ACCOUNTS).put(scalarToken, user.userId, 30 * 60 * 1000); // 30 minutes
                return user.userId;
            }
            catch (err) {
                matrix_js_snippets_1.LogService.error("ScalarService", err);
                throw new ApiError_1.ApiError(401, "Invalid token");
            }
        });
    }
    /**
     * Registers an account to use the Integration Manager
     * @param {OpenId} openId The OpenID request information.
     * @param {string} scalarKind The kind of scalar client to use.
     * @returns {Promise<IAccountRegisteredResponse>} Resolves when registered.
     */
    registerAccount(openId, scalarKind) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!openId || !openId.matrix_server_name || !openId.access_token) {
                throw new ApiError_1.ApiError(400, "Missing OpenID information");
            }
            const mxClient = new MatrixOpenIdClient_1.MatrixOpenIdClient(openId);
            const mxUserId = yield mxClient.getUserId();
            if (!mxUserId.endsWith(":" + openId.matrix_server_name)) {
                matrix_js_snippets_1.LogService.warn("AccountController", `OpenID subject '${mxUserId}' does not belong to the homeserver '${openId.matrix_server_name}'`);
                throw new ApiError_1.ApiError(401, "Invalid token");
            }
            const user = yield User_1.default.findByPk(mxUserId);
            if (!user) {
                // There's a small chance we'll get a validation error because of:
                // https://github.com/vector-im/riot-web/issues/5846
                matrix_js_snippets_1.LogService.verbose("AccountController", "User " + mxUserId + " never seen before - creating");
                yield User_1.default.create({ userId: mxUserId });
            }
            const upstreams = yield Upstream_1.default.findAll();
            yield Promise.all(upstreams.map((upstream) => __awaiter(this, void 0, void 0, function* () {
                if (!(yield ScalarStore_1.ScalarStore.isUpstreamOnline(upstream, scalarKind))) {
                    matrix_js_snippets_1.LogService.warn("AccountController", `Skipping registration for ${mxUserId} on upstream ${upstream.id} (${upstream.name}) because it is offline`);
                    return null;
                }
                const tokens = yield UserScalarToken_1.default.findAll({ where: { userId: mxUserId, upstreamId: upstream.id } });
                if (!tokens || tokens.length === 0) {
                    matrix_js_snippets_1.LogService.info("AccountController", "Registering " + mxUserId + " for a token at upstream " + upstream.id + " (" + upstream.name + ")");
                    const client = new ScalarClient_1.ScalarClient(upstream);
                    const response = yield client.register(openId);
                    return UserScalarToken_1.default.create({
                        userId: mxUserId,
                        scalarToken: response.scalar_token,
                        isDimensionToken: false,
                        upstreamId: upstream.id,
                    });
                }
            })).filter(token => !!token));
            const dimensionToken = randomString({ length: 25 });
            const dimensionScalarToken = yield UserScalarToken_1.default.create({
                userId: mxUserId,
                scalarToken: dimensionToken,
                isDimensionToken: true,
            });
            matrix_js_snippets_1.LogService.info("AccountController", mxUserId + " has registered for a scalar token successfully");
            return { token: dimensionScalarToken.scalarToken };
        });
    }
    /**
     * Logs a user out
     * @param {ILoggedInUser} user The user to log out
     * @returns {Promise<*>} Resolves when complete.
     */
    logout(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const tokens = yield UserScalarToken_1.default.findAll({ where: { scalarToken: user.token }, include: [Upstream_1.default] });
            for (const token of tokens) {
                if (token.upstream) {
                    matrix_js_snippets_1.LogService.info("AccountController", "Logging user out of upstream");
                    const client = new ScalarClient_1.ScalarClient(token.upstream, ScalarClient_1.ScalarClient.KIND_MATRIX_V1);
                    yield client.logout(token.scalarToken);
                }
                yield token.destroy();
            }
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SCALAR_ACCOUNTS).clear();
            return {};
        });
    }
};
AccountController = __decorate([
    es6_1.AutoWired,
    __metadata("design:paramtypes", [])
], AccountController);
exports.default = AccountController;
//# sourceMappingURL=AccountController.js.map