"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ApiError_1 = require("../ApiError");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const AccountController_1 = require("../controllers/AccountController");
const TermsController_1 = require("../controllers/TermsController");
const config_1 = require("../../config");
const ScalarStore_1 = require("../../db/ScalarStore");
const ScalarClient_1 = require("../../scalar/ScalarClient");
exports.ROLE_USER = "ROLE_USER";
exports.ROLE_ADMIN = "ROLE_ADMIN";
const TERMS_IGNORED_ROUTES = [
    { method: "*", path: "/api/v1/dimension/admin/" },
    { method: "GET", path: "/_matrix/integrations/v1/terms" },
    { method: "POST", path: "/_matrix/integrations/v1/terms" },
    { method: "POST", path: "/_matrix/integrations/v1/register" },
    { method: "POST", path: "/_matrix/integrations/v1/logout" },
    // Legacy scalar routes
    { method: "GET", path: "/api/v1/scalar/terms" },
    { method: "POST", path: "/api/v1/scalar/terms" },
    { method: "POST", path: "/api/v1/scalar/register" },
];
const ADMIN_ROUTES = [
    { method: "*", path: "/api/v1/dimension/admin/" },
];
class MatrixSecurity {
    constructor() {
        this.accountController = new AccountController_1.default();
        this.termsController = new TermsController_1.default();
    }
    getRoles(req) {
        if (req.user) {
            const roles = [exports.ROLE_USER];
            if (config_1.default.admins.includes(req.user.userId)) {
                roles.push(exports.ROLE_ADMIN);
            }
            return roles;
        }
        return [];
    }
    getMiddleware() {
        return ((req, res, next) => __awaiter(this, void 0, void 0, function* () {
            try {
                let token = null;
                if (req.headers.authorization) {
                    const header = req.headers.authorization;
                    if (!header.startsWith("Bearer ")) {
                        return res.status(401).json({ errcode: "M_INVALID_TOKEN", error: "Invalid token" });
                    }
                    token = header.substring("Bearer ".length);
                }
                else if (req.query && req.query.access_token) {
                    token = req.query.access_token;
                }
                else if (req.query && req.query.scalar_token) {
                    matrix_js_snippets_1.LogService.warn("MatrixSecurity", "Request used old scalar_token auth - this will be removed in a future version");
                    token = req.query.scalar_token;
                }
                if (token) {
                    req.user = {
                        userId: yield this.accountController.getTokenOwner(token),
                        token: token,
                    };
                    const needUpstreams = !this.matchesAnyRoute(req, ADMIN_ROUTES);
                    if (needUpstreams) {
                        const scalarKind = req.path.startsWith("/_matrix/integrations/v1/") ? ScalarClient_1.ScalarClient.KIND_MATRIX_V1 : ScalarClient_1.ScalarClient.KIND_LEGACY;
                        const hasUpstreams = yield ScalarStore_1.ScalarStore.doesUserHaveTokensForAllUpstreams(req.user.userId, scalarKind);
                        if (!hasUpstreams) {
                            return res.status(401).json({ errcode: "M_INVALID_TOKEN", error: "Invalid token" });
                        }
                    }
                    const needTerms = !this.matchesAnyRoute(req, TERMS_IGNORED_ROUTES);
                    if (needTerms) {
                        const signatureNeeded = yield this.termsController.doesUserNeedToSignTerms(req.user);
                        if (signatureNeeded) {
                            return res.status(403).json({
                                errcode: "M_TERMS_NOT_SIGNED",
                                error: "The user has not accepted all terms of service for this integration manager",
                            });
                        }
                    }
                    if (this.matchesAnyRoute(req, ADMIN_ROUTES, false) && !this.getRoles(req).includes(exports.ROLE_ADMIN)) {
                        return res.status(403).json({ errcode: "M_UNAUTHORIZED", error: "User is not an admin" });
                    }
                    return next();
                }
                else {
                    return res.status(401).json({ errcode: "M_INVALID_TOKEN", error: "Invalid token" });
                }
            }
            catch (e) {
                if (e instanceof ApiError_1.ApiError) {
                    res.status(e.statusCode).json(e.jsonResponse);
                }
                else {
                    matrix_js_snippets_1.LogService.error("MatrixSecurity", e);
                    res.status(500).json({ errcode: "M_UNKNOWN", error: "Unknown server error" });
                }
            }
        }));
    }
    initialize(_router) {
    }
    matchesAnyRoute(req, routes, valForOptions = true) {
        if (req.method === 'OPTIONS')
            return valForOptions;
        for (const route of routes) {
            if (route.method === '*' && req.path.startsWith(route.path)) {
                return true;
            }
            if (route.method === req.method && route.path === req.path) {
                return true;
            }
        }
        return false;
    }
}
exports.default = MatrixSecurity;
//# sourceMappingURL=MatrixSecurity.js.map