"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const MemoryCache_1 = require("../MemoryCache");
const IrcBridgeRecord_1 = require("../db/models/IrcBridgeRecord");
const Upstream_1 = require("../db/models/Upstream");
const UserScalarToken_1 = require("../db/models/UserScalarToken");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const request = require("request");
const IrcBridgeNetwork_1 = require("../db/models/IrcBridgeNetwork");
class IrcBridge {
    constructor(requestingUserId) {
        this.requestingUserId = requestingUserId;
    }
    static getNetworkId(network) {
        return network.ircBridgeId + "-" + network.bridgeNetworkId;
    }
    static parseNetworkId(networkId) {
        const parts = networkId.split("-");
        const bridgeId = parts.splice(0, 1)[0];
        const bridgeNetworkId = parts.join("-");
        return { bridgeId, bridgeNetworkId };
    }
    isBridgingEnabled() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield IrcBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            return !!bridges && bridges.length > 0;
        });
    }
    hasNetworks() {
        return __awaiter(this, void 0, void 0, function* () {
            const allNetworks = (yield this.getAllNetworks()).filter(n => n.isEnabled);
            return allNetworks.length > 0;
        });
    }
    getNetworks(bridge, enabledOnly) {
        return __awaiter(this, void 0, void 0, function* () {
            let networks = yield this.getAllNetworks();
            if (bridge)
                networks = networks.filter(n => n.ircBridgeId === bridge.id);
            if (enabledOnly)
                networks = networks.filter(n => n.isEnabled);
            const available = {};
            networks.forEach(n => available[IrcBridge.getNetworkId(n)] = {
                name: n.displayName,
                domain: n.domain,
                bridgeUserId: n.bridgeUserId,
                isEnabled: n.isEnabled,
            });
            return available;
        });
    }
    getRoomConfiguration(inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const availableNetworks = yield this.getNetworks(null, true);
            const bridges = yield IrcBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            const linkedChannels = {};
            for (const bridge of bridges) {
                const links = yield this.fetchLinks(bridge, inRoomId);
                for (const key of Object.keys(links)) {
                    linkedChannels[key] = links[key];
                }
            }
            return { availableNetworks: availableNetworks, links: linkedChannels };
        });
    }
    getOperators(bridge, networkId, channel) {
        return __awaiter(this, void 0, void 0, function* () {
            const network = (yield this.getAllNetworks()).find(n => n.isEnabled && n.ircBridgeId === bridge.id && n.bridgeNetworkId === networkId);
            if (!network)
                throw new Error("Network not found");
            const requestBody = { remote_room_server: network.domain, remote_room_channel: channel };
            let responses = [];
            if (bridge.upstreamId) {
                const result = yield this.doUpstreamRequest(bridge, "POST", "/bridges/irc/_matrix/provision/querylink", null, requestBody);
                if (result && result.replies)
                    responses = result.replies.map(r => r.response);
            }
            else {
                const result = yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/querylink", null, requestBody);
                if (result)
                    responses = [result];
            }
            const ops = [];
            for (const response of responses) {
                if (!response || !response.operators)
                    continue;
                response.operators.forEach(i => ops.push(i));
            }
            return ops;
        });
    }
    requestLink(bridge, networkId, channel, op, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const network = (yield this.getAllNetworks()).find(n => n.isEnabled && n.ircBridgeId === bridge.id && n.bridgeNetworkId === networkId);
            if (!network)
                throw new Error("Network not found");
            const requestBody = {
                remote_room_server: network.domain,
                remote_room_channel: channel,
                matrix_room_id: inRoomId,
                op_nick: op,
                user_id: this.requestingUserId,
            };
            if (bridge.upstreamId) {
                delete requestBody["user_id"];
                yield this.doUpstreamRequest(bridge, "POST", "/bridges/irc/_matrix/provision/link", null, requestBody);
            }
            else {
                yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/link", null, requestBody);
            }
        });
    }
    removeLink(bridge, networkId, channel, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const network = (yield this.getAllNetworks()).find(n => n.isEnabled && n.ircBridgeId === bridge.id && n.bridgeNetworkId === networkId);
            if (!network)
                throw new Error("Network not found");
            const requestBody = {
                remote_room_server: network.domain,
                remote_room_channel: channel,
                matrix_room_id: inRoomId,
                user_id: this.requestingUserId,
            };
            if (bridge.upstreamId) {
                delete requestBody["user_id"];
                yield this.doUpstreamRequest(bridge, "POST", "/bridges/irc/_matrix/provision/unlink", null, requestBody);
            }
            else {
                yield this.doProvisionRequest(bridge, "POST", "/_matrix/provision/unlink", null, requestBody);
            }
        });
    }
    getAllNetworks() {
        return __awaiter(this, void 0, void 0, function* () {
            const cached = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_IRC_BRIDGE).get("networks");
            if (cached)
                return cached;
            const bridges = yield IrcBridgeRecord_1.default.findAll();
            if (!bridges)
                return [];
            const networks = [];
            for (const bridge of bridges) {
                const bridgeNetworks = yield this.fetchNetworks(bridge);
                bridgeNetworks.forEach(n => networks.push(n));
            }
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_IRC_BRIDGE).put("networks", networks, 60 * 60 * 1000); // 1 hour
            return networks;
        });
    }
    fetchLinks(bridge, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const availableNetworks = yield this.getNetworks(bridge, true);
            const networksByDomain = {};
            for (const key of Object.keys(availableNetworks)) {
                const network = availableNetworks[key];
                networksByDomain[network.domain] = {
                    id: key,
                    name: network.name,
                    bridgeUserId: network.bridgeUserId,
                };
            }
            let responses = [];
            if (bridge.upstreamId) {
                const result = yield this.doUpstreamRequest(bridge, "GET", "/bridges/irc/_matrix/provision/listlinks/" + inRoomId);
                if (result && result.replies) {
                    const replies = result.replies.map(r => r.response);
                    for (const reply of replies)
                        reply.forEach(r => responses.push(r));
                }
            }
            else {
                const result = yield this.doProvisionRequest(bridge, "GET", "/_matrix/provision/listlinks/" + inRoomId);
                if (result)
                    responses = result;
            }
            const linked = {};
            for (const response of responses) {
                if (!response || !response.remote_room_server)
                    continue;
                const network = networksByDomain[response.remote_room_server];
                if (!network)
                    continue;
                if (!linked[network.id])
                    linked[network.id] = [];
                linked[network.id].push({
                    channelName: response.remote_room_channel,
                });
            }
            return linked;
        });
    }
    fetchNetworks(bridge) {
        return __awaiter(this, void 0, void 0, function* () {
            let responses = [];
            if (bridge.upstreamId) {
                const result = yield this.doUpstreamRequest(bridge, "GET", "/bridges/irc/_matrix/provision/querynetworks");
                if (result && result.replies)
                    responses = result.replies.map(r => r.response);
            }
            else {
                const result = yield this.doProvisionRequest(bridge, "GET", "/_matrix/provision/querynetworks");
                if (result)
                    responses = [result];
            }
            const networks = [];
            for (const response of responses) {
                if (!response || !response.servers)
                    continue;
                for (const server of response.servers) {
                    if (!server)
                        continue;
                    let existingNetwork = yield IrcBridgeNetwork_1.default.findOne({
                        where: {
                            bridgeId: bridge.id,
                            bridgeNetworkId: server.network_id,
                        },
                    });
                    if (!existingNetwork) {
                        matrix_js_snippets_1.LogService.info("IrcBridge", "Discovered new network for bridge " + bridge.id + ": " + server.network_id);
                        existingNetwork = yield IrcBridgeNetwork_1.default.create({
                            bridgeId: bridge.id,
                            isEnabled: false,
                            bridgeNetworkId: server.network_id,
                            bridgeUserId: server.bot_user_id,
                            displayName: server.desc,
                            domain: server.fields.domain,
                        });
                    }
                    else {
                        existingNetwork.displayName = server.desc;
                        existingNetwork.bridgeUserId = server.bot_user_id;
                        existingNetwork.domain = server.fields.domain;
                        yield existingNetwork.save();
                    }
                    networks.push({
                        ircBridgeId: bridge.id,
                        bridgeNetworkId: existingNetwork.bridgeNetworkId,
                        bridgeUserId: existingNetwork.bridgeUserId,
                        displayName: existingNetwork.displayName,
                        domain: existingNetwork.domain,
                        isEnabled: existingNetwork.isEnabled,
                    });
                }
            }
            return networks;
        });
    }
    doUpstreamRequest(bridge, method, endpoint, qs, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const upstream = yield Upstream_1.default.findByPk(bridge.upstreamId);
            const token = yield UserScalarToken_1.default.findOne({
                where: {
                    upstreamId: upstream.id,
                    isDimensionToken: false,
                    userId: this.requestingUserId,
                },
            });
            if (!qs)
                qs = {};
            qs["scalar_token"] = token.scalarToken;
            const apiUrl = upstream.apiUrl.endsWith("/") ? upstream.apiUrl.substring(0, upstream.apiUrl.length - 1) : upstream.apiUrl;
            const url = apiUrl + (endpoint.startsWith("/") ? endpoint : "/" + endpoint);
            matrix_js_snippets_1.LogService.info("IrcBridge", "Doing upstream IRC Bridge request: " + url);
            return new Promise((resolve, reject) => {
                request({
                    method: method,
                    url: url,
                    qs: qs,
                    json: body,
                }, (err, res, _body) => {
                    try {
                        if (err) {
                            matrix_js_snippets_1.LogService.error("IrcBridge", "Error calling " + url);
                            matrix_js_snippets_1.LogService.error("IrcBridge", err);
                            reject(err);
                        }
                        else if (!res) {
                            matrix_js_snippets_1.LogService.error("IrcBridge", "There is no response for " + url);
                            reject(new Error("No response provided - is the service online?"));
                        }
                        else if (res.statusCode !== 200) {
                            matrix_js_snippets_1.LogService.error("IrcBridge", "Got status code " + res.statusCode + " when calling " + url);
                            matrix_js_snippets_1.LogService.error("IrcBridge", res.body);
                            reject(new Error("Request failed"));
                        }
                        else {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            resolve(res.body);
                        }
                    }
                    catch (e) {
                        matrix_js_snippets_1.LogService.error("IrcBridge", e);
                        reject(e);
                    }
                });
            });
        });
    }
    doProvisionRequest(bridge, method, endpoint, qs, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const provisionUrl = bridge.provisionUrl;
            const apiUrl = provisionUrl.endsWith("/") ? provisionUrl.substring(0, provisionUrl.length - 1) : provisionUrl;
            const url = apiUrl + (endpoint.startsWith("/") ? endpoint : "/" + endpoint);
            matrix_js_snippets_1.LogService.info("IrcBridge", "Doing provision IRC Bridge request: " + url);
            return new Promise((resolve, reject) => {
                request({
                    method: method,
                    url: url,
                    qs: qs,
                    json: body,
                }, (err, res, _body) => {
                    try {
                        if (err) {
                            matrix_js_snippets_1.LogService.error("IrcBridge", "Error calling" + url);
                            matrix_js_snippets_1.LogService.error("IrcBridge", err);
                            reject(err);
                        }
                        else if (!res) {
                            matrix_js_snippets_1.LogService.error("IrcBridge", "There is no response for " + url);
                            reject(new Error("No response provided - is the service online?"));
                        }
                        else if (res.statusCode !== 200) {
                            matrix_js_snippets_1.LogService.error("IrcBridge", "Got status code " + res.statusCode + " when calling " + url);
                            matrix_js_snippets_1.LogService.error("IrcBridge", res.body);
                            reject(new Error("Request failed"));
                        }
                        else {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            resolve(res.body);
                        }
                    }
                    catch (e) {
                        matrix_js_snippets_1.LogService.error("IrcBridge", e);
                        reject(e);
                    }
                });
            });
        });
    }
}
exports.IrcBridge = IrcBridge;
//# sourceMappingURL=IrcBridge.js.map