"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const TelegramBridgeRecord_1 = require("../db/models/TelegramBridgeRecord");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const request = require("request");
class TelegramBridge {
    constructor(requestingUserId) {
        this.requestingUserId = requestingUserId;
    }
    getDefaultBridge() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield TelegramBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            if (!bridges || bridges.length !== 1) {
                throw new Error("No bridges or too many bridges found");
            }
            return bridges[0];
        });
    }
    isBridgingEnabled() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridges = yield TelegramBridgeRecord_1.default.findAll({ where: { isEnabled: true } });
            return !!bridges && bridges.length > 0;
        });
    }
    getBridgeInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const info = yield this.doProvisionRequest(bridge, "GET", `/bridge`);
            return {
                botUsername: info.relaybot_username,
            };
        });
    }
    getPuppetInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            const info = yield this.doProvisionRequest(bridge, "GET", `/user/${this.requestingUserId}`);
            const puppet = {
                advertiseTgPuppets: bridge.allowTgPuppets,
                advertiseMxPuppets: bridge.allowMxPuppets,
                linked: !!info.telegram,
                telegram: {
                    permissions: [info.permissions],
                    username: info.telegram ? info.telegram.username : null,
                    firstName: info.telegram ? info.telegram.first_name : null,
                    lastName: info.telegram ? info.telegram.last_name : null,
                    phone: info.telegram ? info.telegram.phone : null,
                    isBot: info.telegram ? info.telegram.is_bot : null,
                    availableChats: [],
                },
            };
            if (puppet.linked) {
                puppet.telegram.availableChats = yield this.doProvisionRequest(bridge, "GET", `/user/${this.requestingUserId}/chats`);
            }
            return puppet;
        });
    }
    getRoomConfiguration(inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            try {
                const info = yield this.doProvisionRequest(bridge, "GET", `/portal/${inRoomId}`);
                return {
                    bridged: !!info && info.mxid === inRoomId,
                    chatId: info ? info.chat_id : 0,
                    roomId: info.mxid,
                    chatName: info ? info.title || info.username : null,
                    canUnbridge: info ? info.can_unbridge : false,
                };
            }
            catch (e) {
                if (!e.errBody || e.errBody["errcode"] !== "portal_not_found") {
                    throw e.error || e;
                }
                return {
                    bridged: false,
                    chatId: 0,
                    roomId: "",
                    chatName: null,
                    canUnbridge: false,
                };
            }
        });
    }
    getChatConfiguration(chatId, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            try {
                const info = yield this.doProvisionRequest(bridge, "GET", `/portal/${chatId}`, { room_id: roomId });
                return {
                    bridged: info && !!info.mxid,
                    chatId: chatId,
                    roomId: info ? info.mxid : null,
                    chatName: info ? info.title || info.username : null,
                    canUnbridge: info ? info.can_unbridge : false,
                };
            }
            catch (e) {
                if (!e.errBody || e.errBody["errcode"] !== "portal_not_found") {
                    throw e.error || e;
                }
                const rethrowCodes = ["bot_not_in_chat"];
                if (rethrowCodes.indexOf(e.errBody["errcode"]) !== -1) {
                    throw { errcode: e.errBody["errcode"] };
                }
                return {
                    bridged: false,
                    chatId: 0,
                    roomId: null,
                    chatName: null,
                    canUnbridge: false,
                };
            }
        });
    }
    bridgeRoom(chatId, roomId, unbridgeOtherPortals = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            try {
                const qs = {};
                if (unbridgeOtherPortals)
                    qs["force"] = "unbridge";
                yield this.doProvisionRequest(bridge, "POST", `/portal/${roomId}/connect/${chatId}`, qs);
                return this.getChatConfiguration(chatId, roomId);
            }
            catch (e) {
                if (!e.errBody)
                    throw e.error || e;
                const rethrowCodes = ["not_enough_permissions", "room_already_bridged", "chat_already_bridged", "bot_not_in_chat"];
                if (rethrowCodes.indexOf(e.errBody["errcode"]) !== -1) {
                    throw { errcode: e.errBody["errcode"] };
                }
                throw e.error || e;
            }
        });
    }
    unbridgeRoom(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield this.getDefaultBridge();
            try {
                yield this.doProvisionRequest(bridge, "POST", `/portal/${roomId}/disconnect`);
                return this.getRoomConfiguration(roomId);
            }
            catch (e) {
                if (!e.errBody)
                    throw e.error || e;
                const rethrowCodes = ["not_enough_permissions", "bot_not_in_chat"];
                if (rethrowCodes.indexOf(e.errBody["errcode"]) !== -1) {
                    throw { errcode: e.errBody["errcode"] };
                }
                throw e.error || e;
            }
        });
    }
    doProvisionRequest(bridge, method, endpoint, qs, body) {
        return __awaiter(this, void 0, void 0, function* () {
            const provisionUrl = bridge.provisionUrl;
            const apiUrl = provisionUrl.endsWith("/") ? provisionUrl.substring(0, provisionUrl.length - 1) : provisionUrl;
            const url = apiUrl + (endpoint.startsWith("/") ? endpoint : "/" + endpoint);
            matrix_js_snippets_1.LogService.info("TelegramBridge", "Doing provision Telegram Bridge request: " + url);
            if (!qs)
                qs = {};
            if (qs["user_id"] === false)
                delete qs["user_id"];
            else if (!qs["user_id"])
                qs["user_id"] = this.requestingUserId;
            return new Promise((resolve, reject) => {
                request({
                    method: method,
                    url: url,
                    qs: qs,
                    json: body,
                    headers: {
                        "Authorization": `Bearer ${bridge.sharedSecret}`,
                    },
                }, (err, res, _body) => {
                    try {
                        if (err) {
                            matrix_js_snippets_1.LogService.error("TelegramBridge", "Error calling" + url);
                            matrix_js_snippets_1.LogService.error("TelegramBridge", err);
                            reject(err);
                        }
                        else if (!res) {
                            matrix_js_snippets_1.LogService.error("TelegramBridge", "There is no response for " + url);
                            reject(new Error("No response provided - is the service online?"));
                        }
                        else if (res.statusCode !== 200 && res.statusCode !== 202) {
                            matrix_js_snippets_1.LogService.error("TelegramBridge", "Got status code " + res.statusCode + " when calling " + url);
                            matrix_js_snippets_1.LogService.error("TelegramBridge", res.body);
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            reject({ errBody: res.body, error: new Error("Request failed") });
                        }
                        else {
                            if (typeof (res.body) === "string")
                                res.body = JSON.parse(res.body);
                            resolve(res.body);
                        }
                    }
                    catch (e) {
                        matrix_js_snippets_1.LogService.error("TelegramBridge", e);
                        reject(e);
                    }
                });
            });
        });
    }
}
exports.TelegramBridge = TelegramBridge;
//# sourceMappingURL=TelegramBridge.js.map