"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Bridge_1 = require("../integrations/Bridge");
const BridgeRecord_1 = require("./models/BridgeRecord");
const IrcBridge_1 = require("../bridges/IrcBridge");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const TelegramBridge_1 = require("../bridges/TelegramBridge");
const WebhooksBridge_1 = require("../bridges/WebhooksBridge");
const GitterBridge_1 = require("../bridges/GitterBridge");
const SlackBridge_1 = require("../bridges/SlackBridge");
class BridgeStore {
    static listAll(requestingUserId, isEnabled, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            let conditions = {};
            if (isEnabled === true || isEnabled === false)
                conditions = { where: { isEnabled: isEnabled } };
            const allRecords = yield BridgeRecord_1.default.findAll(conditions);
            const enabledBridges = [];
            for (const bridgeRecord of allRecords) {
                matrix_js_snippets_1.LogService.info("BridgeStore", "Checking bridge configuration: " + bridgeRecord.name);
                let isLogicallyEnabled = yield BridgeStore.hasBridgesConfigured(bridgeRecord, requestingUserId);
                try {
                    if (isEnabled === true || isEnabled === false) {
                        isLogicallyEnabled = yield BridgeStore.isLogicallyEnabled(bridgeRecord, requestingUserId);
                        if (isLogicallyEnabled !== isEnabled)
                            continue;
                    }
                    const bridgeConfig = yield BridgeStore.getConfiguration(bridgeRecord, requestingUserId, inRoomId);
                    enabledBridges.push(new Bridge_1.Bridge(bridgeRecord, bridgeConfig));
                }
                catch (e) {
                    // Skip bridges which just aren't online
                    if (!isLogicallyEnabled)
                        continue;
                    matrix_js_snippets_1.LogService.error("BridgeStore", "Failed to load configuration for bridge: " + bridgeRecord.name);
                    matrix_js_snippets_1.LogService.error("BridgeStore", e);
                    const bridge = new Bridge_1.Bridge(bridgeRecord, {});
                    bridge.isOnline = false;
                    enabledBridges.push(bridge);
                }
            }
            return enabledBridges;
        });
    }
    static setEnabled(type, isEnabled) {
        return __awaiter(this, void 0, void 0, function* () {
            const bridge = yield BridgeRecord_1.default.findOne({ where: { type: type } });
            if (!bridge)
                throw new Error("Bridge not found");
            bridge.isEnabled = isEnabled;
            return bridge.save();
        });
    }
    static setBridgeRoomConfig(_requestingUserId, integrationType, _inRoomId, _newConfig) {
        return __awaiter(this, void 0, void 0, function* () {
            const record = yield BridgeRecord_1.default.findOne({ where: { type: integrationType } });
            if (!record)
                throw new Error("Bridge not found");
            const hasDedicatedApi = ["irc", "telegram", "webhooks", "gitter", "slack"];
            if (hasDedicatedApi.indexOf(integrationType) !== -1) {
                throw new Error("This bridge should be modified with the dedicated API");
            }
            else
                throw new Error("Unsupported bridge");
        });
    }
    static isLogicallyEnabled(record, requestingUserId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (record.type === "irc") {
                const irc = new IrcBridge_1.IrcBridge(requestingUserId);
                return irc.hasNetworks();
            }
            else if (record.type === "telegram") {
                const telegram = new TelegramBridge_1.TelegramBridge(requestingUserId);
                return telegram.isBridgingEnabled();
            }
            else if (record.type === "webhooks") {
                const webhooks = new WebhooksBridge_1.WebhooksBridge(requestingUserId);
                return webhooks.isBridgingEnabled();
            }
            else if (record.type === "gitter") {
                const gitter = new GitterBridge_1.GitterBridge(requestingUserId);
                return gitter.isBridgingEnabled();
            }
            else if (record.type === "slack") {
                const slack = new SlackBridge_1.SlackBridge(requestingUserId);
                return slack.isBridgingEnabled();
            }
            else
                return true;
        });
    }
    static hasBridgesConfigured(record, requestingUserId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (record.type === "irc") {
                const irc = new IrcBridge_1.IrcBridge(requestingUserId);
                return irc.isBridgingEnabled();
            }
            else if (record.type === "telegram") {
                const telegram = new TelegramBridge_1.TelegramBridge(requestingUserId);
                return telegram.isBridgingEnabled();
            }
            else if (record.type === "webhooks") {
                const webhooks = new WebhooksBridge_1.WebhooksBridge(requestingUserId);
                return webhooks.isBridgingEnabled();
            }
            else if (record.type === "gitter") {
                const gitter = new GitterBridge_1.GitterBridge(requestingUserId);
                return gitter.isBridgingEnabled();
            }
            else if (record.type === "slack") {
                const slack = new SlackBridge_1.SlackBridge(requestingUserId);
                return slack.isBridgingEnabled();
            }
            else
                return false;
        });
    }
    static getConfiguration(record, requestingUserId, inRoomId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (record.type === "irc") {
                if (!inRoomId)
                    return {}; // The bridge's admin config is handled by other APIs
                const irc = new IrcBridge_1.IrcBridge(requestingUserId);
                return irc.getRoomConfiguration(inRoomId);
            }
            else if (record.type === "telegram") {
                if (!inRoomId)
                    return {}; // The bridge's admin config is handled by other APIs
                const telegram = new TelegramBridge_1.TelegramBridge(requestingUserId);
                const roomConf = yield telegram.getRoomConfiguration(inRoomId);
                const bridgeInfo = yield telegram.getBridgeInfo();
                return {
                    botUsername: bridgeInfo.botUsername,
                    linked: roomConf.bridged ? [roomConf.chatId] : [],
                    portalInfo: roomConf,
                    puppet: yield telegram.getPuppetInfo(),
                };
            }
            else if (record.type === "webhooks") {
                if (!inRoomId)
                    return {}; // The bridge's admin config is handled by other APIs
                const webhooks = new WebhooksBridge_1.WebhooksBridge(requestingUserId);
                const hooks = yield webhooks.getHooks(inRoomId);
                const info = yield webhooks.getBridgeInfo();
                return {
                    webhooks: hooks,
                    botUserId: info.botUserId,
                };
            }
            else if (record.type === "gitter") {
                if (!inRoomId)
                    return {}; // The bridge's admin config is handled by other APIs
                const gitter = new GitterBridge_1.GitterBridge(requestingUserId);
                const info = yield gitter.getBridgeInfo();
                const link = yield gitter.getLink(inRoomId);
                return {
                    link: link,
                    botUserId: info.botUserId,
                };
            }
            else if (record.type === "slack") {
                if (!inRoomId)
                    return {}; // The bridge's admin config is handled by other APIs
                const slack = new SlackBridge_1.SlackBridge(requestingUserId);
                const info = yield slack.getBridgeInfo();
                const link = yield slack.getLink(inRoomId);
                return {
                    link: link,
                    botUserId: info.botUserId,
                };
            }
            else
                return {};
        });
    }
    constructor() {
    }
}
exports.BridgeStore = BridgeStore;
//# sourceMappingURL=BridgeStore.js.map