"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const UserScalarToken_1 = require("./models/UserScalarToken");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const Upstream_1 = require("./models/Upstream");
const User_1 = require("./models/User");
const MatrixStickerBot_1 = require("../matrix/MatrixStickerBot");
const ScalarClient_1 = require("../scalar/ScalarClient");
const MemoryCache_1 = require("../MemoryCache");
class ScalarStore {
    static doesUserHaveTokensForAllUpstreams(userId, scalarKind) {
        return __awaiter(this, void 0, void 0, function* () {
            const scalarTokens = yield UserScalarToken_1.default.findAll({ where: { userId: userId } });
            const upstreamTokenIds = scalarTokens.filter(t => !t.isDimensionToken).map(t => t.upstreamId);
            const hasDimensionToken = scalarTokens.filter(t => t.isDimensionToken).length >= 1;
            if (!hasDimensionToken) {
                matrix_js_snippets_1.LogService.warn("ScalarStore", "User " + userId + " is missing a Dimension scalar token");
                return false;
            }
            const upstreams = yield Upstream_1.default.findAll();
            for (const upstream of upstreams) {
                if (!(yield ScalarStore.isUpstreamOnline(upstream, scalarKind))) {
                    matrix_js_snippets_1.LogService.warn("ScalarStore", `Upstream ${upstream.apiUrl} is offline - assuming token is valid`);
                    continue;
                }
                if (upstreamTokenIds.indexOf(upstream.id) === -1) {
                    matrix_js_snippets_1.LogService.warn("ScalarStore", "user " + userId + " is missing a scalar token for upstream " + upstream.id + " (" + upstream.name + ")");
                    return false;
                }
            }
            return true;
        });
    }
    static getTokenOwner(scalarToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const tokens = yield UserScalarToken_1.default.findAll({
                where: { isDimensionToken: true, scalarToken: scalarToken },
                include: [User_1.default]
            });
            if (!tokens || tokens.length === 0)
                throw new Error("Invalid token");
            return tokens[0].user;
        });
    }
    static isUpstreamOnline(upstream, scalarKind) {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_SCALAR_ONLINE_STATE);
            const cacheKey = `Upstream ${upstream.id}`;
            const result = cache.get(cacheKey);
            if (typeof (result) === 'boolean') {
                return result;
            }
            const state = ScalarStore.checkIfUpstreamOnline(upstream, scalarKind);
            cache.put(cacheKey, state, 60 * 60 * 1000); // 1 hour
            return state;
        });
    }
    static checkIfUpstreamOnline(upstream, scalarKind, signTerms = true) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // The sticker bot can be used for this for now
                const testUserId = yield MatrixStickerBot_1.MatrixStickerBot.getUserId();
                const scalarClient = new ScalarClient_1.ScalarClient(upstream, scalarKind);
                // First see if we have a token for the upstream so we can try it
                const existingTokens = yield UserScalarToken_1.default.findAll({
                    where: { isDimensionToken: false, userId: testUserId, upstreamId: upstream.id },
                    include: [Upstream_1.default],
                });
                if (existingTokens && existingTokens.length) {
                    // Test that the token works
                    try {
                        const result = yield scalarClient.getAccount(existingTokens[0].scalarToken);
                        if (result.user_id !== testUserId) {
                            // noinspection ExceptionCaughtLocallyJS
                            throw new Error(`Unexpected error: Upstream ${upstream.id} did not return account info for the right token`);
                        }
                        return true; // it's online
                    }
                    catch (e) {
                        matrix_js_snippets_1.LogService.error("ScalarStore", e);
                        if (e && !isNaN(Number(e.statusCode))) {
                            if (e.statusCode === 403 && e.body) {
                                if (e.body.errcode === 'M_TERMS_NOT_SIGNED' && signTerms) {
                                    yield ScalarStore.signAllTerms(existingTokens[0], scalarKind);
                                    return ScalarStore.checkIfUpstreamOnline(upstream, scalarKind, false);
                                }
                            }
                            if (e.statusCode === 401 || e.statusCode === 403) {
                                matrix_js_snippets_1.LogService.info("ScalarStore", "Test user token expired");
                            }
                            else {
                                // Assume offline
                                return false;
                            }
                        }
                    }
                }
                // If we're here, we need to register a new token
                if (existingTokens && existingTokens.length) {
                    for (const token of existingTokens) {
                        yield token.destroy();
                    }
                }
                const user = yield User_1.default.findByPk(testUserId);
                if (!user) {
                    yield User_1.default.create({
                        userId: testUserId,
                        isSelfBot: true,
                    });
                }
                const openId = yield MatrixStickerBot_1.MatrixStickerBot.getOpenId();
                const token = yield scalarClient.register(openId);
                const scalarToken = yield UserScalarToken_1.default.create({
                    userId: testUserId,
                    scalarToken: token.scalar_token,
                    isDimensionToken: false,
                    upstreamId: upstream.id,
                });
                scalarToken.upstream = upstream;
                // Accept all terms of service for the user
                yield ScalarStore.signAllTerms(scalarToken, scalarKind);
                return true;
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("ScalarStore", e);
                return false;
            }
        });
    }
    static signAllTerms(token, scalarKind) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const client = new ScalarClient_1.ScalarClient(token.upstream, scalarKind);
                const terms = yield client.getAvailableTerms();
                const urlsToSign = Object.values(terms.policies).map(p => {
                    const englishCode = Object.keys(p).find(k => k.toLowerCase() === 'en' || k.toLowerCase().startsWith('en_'));
                    if (!englishCode)
                        return null;
                    return p[englishCode].url;
                }).filter(v => !!v);
                yield client.signTermsUrls(token.scalarToken, urlsToSign);
            }
            catch (e) {
                matrix_js_snippets_1.LogService.error("ScalarStore", e);
            }
        });
    }
    constructor() {
    }
}
exports.ScalarStore = ScalarStore;
//# sourceMappingURL=ScalarStore.js.map