"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dns = require("dns-then");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const MemoryCache_1 = require("../MemoryCache");
const request = require("request");
const config_1 = require("../config");
const split_host_1 = require("split-host");
const isIP = require("isipaddress");
const requestPromise = require("request-promise");
function getFederationConnInfo(serverName) {
    return __awaiter(this, void 0, void 0, function* () {
        const expirationMs = 2 * 60 * 60 * 1000; // 2 hours
        // Check to see if we've cached the hostname at all already
        const cachedUrl = MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).get(serverName);
        if (cachedUrl) {
            matrix_js_snippets_1.LogService.verbose("matrix", "Cached federation URL for " + serverName + " is " + cachedUrl.url);
            return cachedUrl;
        }
        // Rely on the configuration for a federation URL if we can
        if (serverName === config_1.default.homeserver.name && config_1.default.homeserver.federationUrl) {
            let url = config_1.default.homeserver.federationUrl;
            if (url.endsWith("/")) {
                url = url.substring(0, url.length - 1);
            }
            matrix_js_snippets_1.LogService.info("matrix", "Using configured federation URL for " + serverName);
            const fedObj = { url, hostname: serverName };
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
            return fedObj;
        }
        // Dev note: The remainder of this is largely transcribed from matrix-media-repo
        const hp = split_host_1.default(serverName);
        if (!hp.host)
            throw new Error("No hostname provided");
        let defaultPort = false;
        if (!hp.port) {
            defaultPort = true;
            hp.port = 8448;
        }
        // Step 1 of the discovery process: if the hostname is an IP, use that with explicit or default port
        if (isIP.test(hp.host)) {
            const fedUrl = `https://${hp.host}:${hp.port}`;
            const fedObj = { url: fedUrl, hostname: serverName };
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
            matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (IP address)`);
            return fedObj;
        }
        // Step 2: if the hostname is not an IP address, and an explicit port is given, use that
        if (!defaultPort) {
            const fedUrl = `https://${hp.host}:${hp.port}`;
            const fedObj = { url: fedUrl, hostname: hp.host };
            MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
            matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (explicit port)`);
            return fedObj;
        }
        // Step 3: if the hostname is not an IP address and no explicit port is given, do .well-known
        try {
            let result = yield requestPromise(`https://${hp.host}/.well-known/matrix/server`);
            if (typeof (result) === 'string')
                result = JSON.parse(result);
            const wkServerAddr = result['m.server'];
            if (wkServerAddr) {
                const wkHp = split_host_1.default(wkServerAddr);
                if (!wkHp.host) {
                    // noinspection ExceptionCaughtLocallyJS
                    throw new Error("No hostname provided for m.server");
                }
                let wkDefaultPort = false;
                if (!wkHp.port) {
                    wkDefaultPort = true;
                    wkHp.port = 8448;
                }
                // Step 3a: if the delegated host is an IP address, use that (regardless of port)
                if (isIP.test(wkHp.host)) {
                    const fedUrl = `https://${wkHp.host}:${wkHp.port}`;
                    const fedObj = { url: fedUrl, hostname: wkServerAddr };
                    MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
                    matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (WK; IP address)`);
                    return fedObj;
                }
                // Step 3b: if the delegated host is not an IP and an explicit port is given, use that
                if (!wkDefaultPort) {
                    const fedUrl = `https://${wkHp.host}:${wkHp.port}`;
                    const fedObj = { url: fedUrl, hostname: wkHp.host };
                    MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
                    matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (WK; explicit port)`);
                    return fedObj;
                }
                // Step 3c: if the delegated host is not an IP and doesn't have a port, start a SRV lookup and use that
                try {
                    const records = yield dns.resolveSrv("_matrix._tcp." + hp.host);
                    if (records && records.length > 0) {
                        const fedUrl = `https://${records[0].name}:${records[0].port}`;
                        const fedObj = { url: fedUrl, hostname: wkHp.host };
                        MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
                        matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (WK; SRV)`);
                        return fedObj;
                    }
                }
                catch (e) {
                    matrix_js_snippets_1.LogService.warn("matrix", "Non-fatal error looking up .well-known SRV for " + serverName);
                    matrix_js_snippets_1.LogService.warn("matrix", e);
                }
                // Step 3d: use the delegated host as-is
                const fedUrl = `https://${wkHp.host}:${wkHp.port}`;
                const fedObj = { url: fedUrl, hostname: wkHp.host };
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
                matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (WK; fallback)`);
                return fedObj;
            }
        }
        catch (e) {
            matrix_js_snippets_1.LogService.warn("matrix", "Non-fatal error looking up .well-known for " + serverName);
            matrix_js_snippets_1.LogService.warn("matrix", e);
        }
        // Step 4: try resolving a hostname using SRV records and use that
        try {
            const records = yield dns.resolveSrv("_matrix._tcp." + hp.host);
            if (records && records.length > 0) {
                const fedUrl = `https://${records[0].name}:${records[0].port}`;
                const fedObj = { url: fedUrl, hostname: hp.host };
                MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
                matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (SRV)`);
                return fedObj;
            }
        }
        catch (e) {
            matrix_js_snippets_1.LogService.warn("matrix", "Non-fatal error looking up SRV for " + serverName);
            matrix_js_snippets_1.LogService.warn("matrix", e);
        }
        // Step 5: use the target host as-is
        const fedUrl = `https://${hp.host}:${hp.port}`;
        const fedObj = { url: fedUrl, hostname: hp.host };
        MemoryCache_1.Cache.for(MemoryCache_1.CACHE_FEDERATION).put(serverName, fedObj, expirationMs);
        matrix_js_snippets_1.LogService.info("matrix", `Federation URL for ${serverName} is ${fedUrl} (SRV)`);
        return fedObj;
    });
}
exports.getFederationConnInfo = getFederationConnInfo;
function doFederatedApiCall(method, serverName, endpoint, query, body) {
    return __awaiter(this, void 0, void 0, function* () {
        const federationInfo = yield getFederationConnInfo(serverName);
        matrix_js_snippets_1.LogService.info("matrix", "Doing federated API call: " + federationInfo.url + endpoint);
        return new Promise((resolve, reject) => {
            request({
                method: method,
                url: federationInfo.url + endpoint,
                qs: query,
                json: body,
                headers: {
                    "Host": federationInfo.hostname,
                },
            }, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("matrix", "Error calling " + endpoint);
                    matrix_js_snippets_1.LogService.error("matrix", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    matrix_js_snippets_1.LogService.error("matrix", "Got status code " + res.statusCode + " while calling federated endpoint " + endpoint);
                    reject(new Error("Error in request: invalid status code"));
                }
                else {
                    if (typeof (res.body) === "string")
                        res.body = JSON.parse(res.body);
                    resolve(res.body);
                }
            });
        });
    });
}
exports.doFederatedApiCall = doFederatedApiCall;
function doClientApiCall(method, endpoint, query, body, contentType = "application/octet-stream") {
    return __awaiter(this, void 0, void 0, function* () {
        let url = config_1.default.homeserver.clientServerUrl;
        if (url.endsWith("/"))
            url = url.substring(0, url.length - 1);
        matrix_js_snippets_1.LogService.info("matrix", "Doing client API call: " + url + endpoint);
        const requestOptions = {
            method: method,
            url: url + endpoint,
            qs: query,
        };
        if (Buffer.isBuffer(body)) {
            requestOptions["body"] = body;
            requestOptions["headers"] = {
                "Content-Type": contentType,
            };
        }
        else {
            requestOptions["json"] = true;
            requestOptions["body"] = body;
        }
        return new Promise((resolve, reject) => {
            request(requestOptions, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("matrix", "Error calling " + endpoint);
                    matrix_js_snippets_1.LogService.error("matrix", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    matrix_js_snippets_1.LogService.error("matrix", "Got status code " + res.statusCode + " while calling client endpoint " + endpoint);
                    matrix_js_snippets_1.LogService.error("matrix", res.body);
                    reject(new Error("Error in request: invalid status code"));
                }
                else {
                    if (typeof (res.body) === "string")
                        res.body = JSON.parse(res.body);
                    resolve(res.body);
                }
            });
        });
    });
}
exports.doClientApiCall = doClientApiCall;
//# sourceMappingURL=helpers.js.map