"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const matrix_js_snippets_1 = require("matrix-js-snippets");
const common_constants_1 = require("../utils/common-constants");
const REGISTER_ROUTE = "/register";
const ACCOUNT_INFO_ROUTE = "/account";
const LOGOUT_ROUTE = "/logout";
const TERMS_ROUTE = "/terms";
class ScalarClient {
    constructor(upstream, kind = ScalarClient.KIND_LEGACY) {
        this.upstream = upstream;
        this.kind = kind;
    }
    makeRequestArguments(path, token) {
        if (this.kind === ScalarClient.KIND_LEGACY) {
            const addlQuery = {};
            if (token)
                addlQuery['scalar_token'] = token;
            return {
                scalarUrl: this.upstream.scalarUrl + path,
                headers: {},
                queryString: Object.assign({ v: common_constants_1.SCALAR_API_VERSION }, addlQuery),
            };
        }
        else {
            const parsed = new URL(this.upstream.scalarUrl);
            if (path === ACCOUNT_INFO_ROUTE || path === TERMS_ROUTE) {
                parsed.pathname = `/_matrix/integrations/v1${path}`;
            }
            else {
                parsed.pathname = `/_matrix/integrations/v1${ACCOUNT_INFO_ROUTE}${path}`;
            }
            const headers = {};
            if (token)
                headers['Authorization'] = `Bearer ${token}`;
            return {
                scalarUrl: parsed.toString(),
                headers: headers,
                queryString: {
                    // HACK: This is due to a Scalar bug
                    // https://github.com/vector-im/riot-web/issues/10498
                    v: common_constants_1.SCALAR_API_VERSION,
                },
            };
        }
    }
    register(openId) {
        const { scalarUrl, headers, queryString } = this.makeRequestArguments(REGISTER_ROUTE, null);
        matrix_js_snippets_1.LogService.info("ScalarClient", "Doing upstream scalar request: " + scalarUrl);
        return new Promise((resolve, reject) => {
            request({
                method: "POST",
                url: scalarUrl,
                qs: queryString,
                headers: headers,
                json: openId,
            }, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Error registering for token");
                    matrix_js_snippets_1.LogService.error("ScalarClient", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Got status code " + res.statusCode + " while registering for token");
                    reject(new Error("Could not get token"));
                }
                else {
                    resolve(res.body);
                }
            });
        });
    }
    getAccount(token) {
        const { scalarUrl, headers, queryString } = this.makeRequestArguments(ACCOUNT_INFO_ROUTE, token);
        matrix_js_snippets_1.LogService.info("ScalarClient", "Doing upstream scalar request: " + scalarUrl);
        return new Promise((resolve, reject) => {
            request({
                method: "GET",
                url: scalarUrl,
                qs: queryString,
                headers: headers,
                json: true,
            }, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Error getting information for token");
                    matrix_js_snippets_1.LogService.error("ScalarClient", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    if (typeof (res.body) === 'string') {
                        try {
                            res.body = JSON.parse(res.body);
                        }
                        catch (e) {
                            matrix_js_snippets_1.LogService.error("ScalarClient", "Got error parsing error response:");
                            matrix_js_snippets_1.LogService.error("ScalarClient", e);
                        }
                    }
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Got status code " + res.statusCode + " while getting information for token");
                    reject(res);
                }
                else {
                    resolve(res.body);
                }
            });
        });
    }
    logout(token) {
        const { scalarUrl, headers, queryString } = this.makeRequestArguments(LOGOUT_ROUTE, token);
        matrix_js_snippets_1.LogService.info("ScalarClient", "Doing upstream scalar request: " + scalarUrl);
        return new Promise((resolve, reject) => {
            request({
                method: "POST",
                url: scalarUrl,
                qs: queryString,
                headers: headers,
                json: true,
            }, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Error logging out token");
                    matrix_js_snippets_1.LogService.error("ScalarClient", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Got status code " + res.statusCode + " while logging out token");
                    reject(res.statusCode);
                }
                else {
                    resolve(res.body);
                }
            });
        });
    }
    getAvailableTerms() {
        const { scalarUrl, headers, queryString } = this.makeRequestArguments(TERMS_ROUTE, null);
        matrix_js_snippets_1.LogService.info("ScalarClient", "Doing upstream scalar request: GET " + scalarUrl);
        return new Promise((resolve, reject) => {
            request({
                method: "GET",
                url: scalarUrl,
                qs: queryString,
                headers: headers,
                json: true,
            }, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Error getting terms for token");
                    matrix_js_snippets_1.LogService.error("ScalarClient", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Got status code " + res.statusCode + " while getting terms for token");
                    reject(res.statusCode);
                }
                else {
                    resolve(res.body);
                }
            });
        });
    }
    signTermsUrls(token, urls) {
        const { scalarUrl, headers, queryString } = this.makeRequestArguments(TERMS_ROUTE, token);
        matrix_js_snippets_1.LogService.info("ScalarClient", "Doing upstream scalar request: POST " + scalarUrl);
        return new Promise((resolve, reject) => {
            request({
                method: "POST",
                url: scalarUrl,
                qs: queryString,
                headers: headers,
                json: { user_accepts: urls },
            }, (err, res, _body) => {
                if (err) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Error updating terms for token");
                    matrix_js_snippets_1.LogService.error("ScalarClient", err);
                    reject(err);
                }
                else if (res.statusCode !== 200) {
                    matrix_js_snippets_1.LogService.error("ScalarClient", "Got status code " + res.statusCode + " while updating terms for token");
                    reject(res.statusCode);
                }
                else {
                    resolve(res.body);
                }
            });
        });
    }
}
ScalarClient.KIND_LEGACY = "legacy";
ScalarClient.KIND_MATRIX_V1 = "matrix_v1";
exports.ScalarClient = ScalarClient;
//# sourceMappingURL=ScalarClient.js.map